/*

	Win32 API definitions used by the SDG toolkit.
	
	Notes:
		- In __sdgMouse::set_Picture you call BeginPaint.  You should port this to GDI+ by calling Graphics.FromControl(this) and using the Graphics.DrawIcon function, instead.
		- Replace sizeof(type) with Marshal.SizeOf(typeof(type))
		- Omit any ZeroMemory calls; they are not needed as the .NET Framework runtime automatically zeroes memory when allocated.
		- Note that there are two overloads for GetRawInputDeviceList.  When you want to pass NULL for the first parameter, pass IntPtr.Zero instead.
		- I have never tried unions before (StructLayout explicit).  There's a good chance these may not work and some fancy footwork will be needed to mitigate the problem.
*/		

using System;
using System.Reflection;
using System.Runtime.InteropServices;


namespace Sdgt
{
	internal class Win32
	{
		[Flags]
		internal enum RI_MOUSE
		{
			RI_MOUSE_LEFT_BUTTON_DOWN = 0x0001,
			RI_MOUSE_LEFT_BUTTON_UP = 0x0002,
			RI_MOUSE_RIGHT_BUTTON_DOWN = 0x0004,
			RI_MOUSE_RIGHT_BUTTON_UP = 0x0008,
			RI_MOUSE_MIDDLE_BUTTON_DOWN = 0x0010,
			RI_MOUSE_MIDDLE_BUTTON_UP = 0x0020,
			RI_MOUSE_BUTTON_1_DOWN = RI_MOUSE_LEFT_BUTTON_DOWN,
			RI_MOUSE_BUTTON_1_UP = RI_MOUSE_LEFT_BUTTON_UP,
			RI_MOUSE_BUTTON_2_DOWN = RI_MOUSE_RIGHT_BUTTON_DOWN,
			RI_MOUSE_BUTTON_2_UP = RI_MOUSE_RIGHT_BUTTON_UP,
			RI_MOUSE_BUTTON_3_DOWN = RI_MOUSE_MIDDLE_BUTTON_DOWN,
			RI_MOUSE_BUTTON_3_UP = RI_MOUSE_MIDDLE_BUTTON_UP,
			RI_MOUSE_BUTTON_4_DOWN = 0x0040,
			RI_MOUSE_BUTTON_4_UP = 0x0080,
			RI_MOUSE_BUTTON_5_DOWN = 0x0100,
			RI_MOUSE_BUTTON_5_UP = 0x0200,
			RI_MOUSE_WHEEL = 0x0400
		}
		
		internal enum SystemSuppliedCursors
		{
			IDC_ARROW = 32512,
			IDC_IBEAM = 32513,
			IDC_WAIT = 32514,
			IDC_CROSS = 32515,
			IDC_UPARROW = 32516,
			IDC_SIZE = 32640,
			IDC_ICON = 32641,
			IDC_SIZENWSE = 32642,
			IDC_SIZENESW = 32643,
			IDC_SIZEWE = 32644,
			IDC_SIZENS = 32645,
			IDC_SIZEALL = 32646,
			IDC_NO = 32648,
			IDC_HAND = 32649,
			IDC_APPSTARTING = 32650,
			IDC_HELP = 32651
		}
		
		internal struct ICONINFO
		{
			[MarshalAs(UnmanagedType.Bool)] bool fIcon;
			public int xHotspot;
			public int yHotspot;
			public IntPtr hbmMask;
			public IntPtr hbmColor;
		}
		
		[Flags]
			internal enum DrawIconFlag
		{
			DI_MASK = 0x0001,
			DI_IMAGE = 0x0002,
			DI_NORMAL = 0x0003,
			DI_COMPAT = 0x0004,
			DI_DEFAULTSIZE = 0x0008,
			DI_NOMIRROR = 0x0010
		}
		
		internal struct RECT
		{
			public int left;
			public int top;
			public int right;
			public int bottom;
		}
		
		internal struct WINDOWINFO
		{
			public int cbSize;
			public RECT rcWindow;
			public RECT rcClient;
			public int dwStyle;
			public int dwExStyle;
			public int dwWindowStatus;
			public int cxWindowBorders;
			public int cyWindowBorders;
			public short atomWindowType;
			public short wCreatorVersion;
		}

		internal struct RAWINPUTDEVICELIST
		{
			public IntPtr hDevice;
			public int dwType;
		}
		
		[Flags]
			internal enum RawInputDeviceFlag
		{
			RIDEV_REMOVE = 0x00000001,
			RIDEV_EXCLUDE = 0x00000010,
			RIDEV_PAGEONLY = 0x00000020,
			RIDEV_NOLEGACY = 0x00000030,
			RIDEV_CAPTUREMOUSE = 0x00000200,
			RIDEV_NOHOTKEYS = 0x00000200,
			RIDEV_EXMODEMASK = 0x000000F0
		}
		
		public static RawInputDeviceFlag RIDEV_EXMODE(RawInputDeviceFlag mode)
		{
			return (RawInputDeviceFlag) (((int) mode) & ((int) RawInputDeviceFlag.RIDEV_EXMODEMASK));
		}
		
		internal struct RAWINPUTDEVICE
		{
			public short usUsagePage;
			public short usUsage;
			[MarshalAs(UnmanagedType.I4)] public RawInputDeviceFlag dwFlags;
			public IntPtr hwndTarget;
		}
		
		internal enum RawInputDeviceType
		{
			RIM_TYPEMOUSE = 0,
			RIM_TYPEKEYBOARD = 1,
			RIM_TYPEHID = 2
		}
		
		internal struct RID_DEVICE_INFO_MOUSE
		{
			public int dwId;
			public int dwNumberOfButtons;
			public int dwSampleRate;
		}
		
		internal struct RID_DEVICE_INFO_KEYBOARD
		{
			public int dwKeyboardType;
			public int dwSubType;
			public int dwKeyboardMode;
			public int dwNumberOfFunctionKeys;
			public int dwNumberOfIndicators;
			public int dwNumberOfKeysTotal;
		}
		
		[StructLayout(LayoutKind.Explicit, Size=32)]
			internal struct RID_DEVICE_INFO
		{
			[FieldOffset(0)] public int cbSize;
			[FieldOffset(4)] public int dwType;
			[FieldOffset(8)] public RID_DEVICE_INFO_MOUSE mouse;
			[FieldOffset(8)] public RID_DEVICE_INFO_KEYBOARD keyboard;
		}
			
		[Flags]
			internal enum VirtKey
		{
			VK_LBUTTON = 0x01,
			VK_RBUTTON = 0x02,
			VK_CANCEL = 0x03,
			VK_MBUTTON = 0x04,
			VK_XBUTTON1 = 0x05,
			VK_XBUTTON2 = 0x06,
			VK_BACK = 0x08,
			VK_TAB = 0x09,
			VK_CLEAR = 0x0C,
			VK_RETURN = 0x0D,
			VK_SHIFT = 0x10,
			VK_CONTROL = 0x11,
			VK_MENU = 0x12,
			VK_PAUSE = 0x13,
			VK_CAPITAL = 0x14,
			VK_KANA = 0x15,
			VK_HANGEUL = 0x15,
			VK_HANGUL = 0x15,
			VK_JUNJA = 0x17,
			VK_FINAL = 0x18,
			VK_HANJA = 0x19,
			VK_KANJI = 0x19,
			VK_ESCAPE = 0x1B,
			VK_CONVERT = 0x1C,
			VK_NONCONVERT = 0x1D,
			VK_ACCEPT = 0x1E,
			VK_MODECHANGE = 0x1F,
			VK_SPACE = 0x20,
			VK_PRIOR = 0x21,
			VK_NEXT = 0x22,
			VK_END = 0x23,
			VK_HOME = 0x24,
			VK_LEFT = 0x25,
			VK_UP = 0x26,
			VK_RIGHT = 0x27,
			VK_DOWN = 0x28,
			VK_SELECT = 0x29,
			VK_PRINT = 0x2A,
			VK_EXECUTE = 0x2B,
			VK_SNAPSHOT = 0x2C,
			VK_INSERT = 0x2D,
			VK_DELETE = 0x2E,
			VK_HELP = 0x2F,
			VK_0 = 0x30,
			VK_1 = 0x31,
			VK_2 = 0x32,
			VK_3 = 0x33,
			VK_4 = 0x34,
			VK_5 = 0x35,
			VK_6 = 0x36,
			VK_7 = 0x37,
			VK_8 = 0x38,
			VK_9 = 0x39,
			VK_A = 0x41,
			VK_B = 0x42,
			VK_C = 0x43,
			VK_D = 0x44,
			VK_E = 0x45,
			VK_F = 0x46,
			VK_G = 0x47,
			VK_H = 0x48,
			VK_I = 0x49,
			VK_J = 0x4A,
			VK_K = 0x4B,
			VK_L = 0x4C,
			VK_M = 0x4D,
			VK_N = 0x4E,
			VK_O = 0x4F,
			VK_P = 0x50,
			VK_Q = 0x51,
			VK_R = 0x52,
			VK_S = 0x53,
			VK_T = 0x54,
			VK_U = 0x55,
			VK_V = 0x56,
			VK_W = 0x57,
			VK_X = 0x58,
			VK_Y = 0x59,
			VK_Z = 0x5A,
			VK_LWIN = 0x5B,
			VK_RWIN = 0x5C,
			VK_APPS = 0x5D,
			VK_SLEEP = 0x5F,
			VK_NUMPAD0 = 0x60,
			VK_NUMPAD1 = 0x61,
			VK_NUMPAD2 = 0x62,
			VK_NUMPAD3 = 0x63,
			VK_NUMPAD4 = 0x64,
			VK_NUMPAD5 = 0x65,
			VK_NUMPAD6 = 0x66,
			VK_NUMPAD7 = 0x67,
			VK_NUMPAD8 = 0x68,
			VK_NUMPAD9 = 0x69,
			VK_MULTIPLY = 0x6A,
			VK_ADD = 0x6B,
			VK_SEPARATOR = 0x6C,
			VK_SUBTRACT = 0x6D,
			VK_DECIMAL = 0x6E,
			VK_DIVIDE = 0x6F,
			VK_F1 = 0x70,
			VK_F2 = 0x71,
			VK_F3 = 0x72,
			VK_F4 = 0x73,
			VK_F5 = 0x74,
			VK_F6 = 0x75,
			VK_F7 = 0x76,
			VK_F8 = 0x77,
			VK_F9 = 0x78,
			VK_F10 = 0x79,
			VK_F11 = 0x7A,
			VK_F12 = 0x7B,
			VK_F13 = 0x7C,
			VK_F14 = 0x7D,
			VK_F15 = 0x7E,
			VK_F16 = 0x7F,
			VK_F17 = 0x80,
			VK_F18 = 0x81,
			VK_F19 = 0x82,
			VK_F20 = 0x83,
			VK_F21 = 0x84,
			VK_F22 = 0x85,
			VK_F23 = 0x86,
			VK_F24 = 0x87,
			VK_NUMLOCK = 0x90,
			VK_SCROLL = 0x91,
			VK_OEM_NEC_EQUAL = 0x92,
			VK_OEM_FJ_JISHO = 0x92,
			VK_OEM_FJ_MASSHOU = 0x93,
			VK_OEM_FJ_TOUROKU = 0x94,
			VK_OEM_FJ_LOYA = 0x95,
			VK_OEM_FJ_ROYA = 0x96,
			VK_LSHIFT = 0xA0,
			VK_RSHIFT = 0xA1,
			VK_LCONTROL = 0xA2,
			VK_RCONTROL = 0xA3,
			VK_LMENU = 0xA4,
			VK_RMENU = 0xA5,
			VK_BROWSER_BACK = 0xA6,
			VK_BROWSER_FORWARD = 0xA7,
			VK_BROWSER_REFRESH = 0xA8,
			VK_BROWSER_STOP = 0xA9,
			VK_BROWSER_SEARCH = 0xAA,
			VK_BROWSER_FAVORITES = 0xAB,
			VK_BROWSER_HOME = 0xAC,
			VK_VOLUME_MUTE = 0xAD,
			VK_VOLUME_DOWN = 0xAE,
			VK_VOLUME_UP = 0xAF,
			VK_MEDIA_NEXT_TRACK = 0xB0,
			VK_MEDIA_PREV_TRACK = 0xB1,
			VK_MEDIA_STOP = 0xB2,
			VK_MEDIA_PLAY_PAUSE = 0xB3,
			VK_LAUNCH_MAIL = 0xB4,
			VK_LAUNCH_MEDIA_SELECT = 0xB5,
			VK_LAUNCH_APP1 = 0xB6,
			VK_LAUNCH_APP2 = 0xB7,
			VK_OEM_1 = 0xBA,
			VK_OEM_PLUS = 0xBB,
			VK_OEM_COMMA = 0xBC,
			VK_OEM_MINUS = 0xBD,
			VK_OEM_PERIOD = 0xBE,
			VK_OEM_2 = 0xBF,
			VK_OEM_3 = 0xC0,
			VK_OEM_4 = 0xDB,
			VK_OEM_5 = 0xDC,
			VK_OEM_6 = 0xDD,
			VK_OEM_7 = 0xDE,
			VK_OEM_8 = 0xDF,
			VK_OEM_AX = 0xE1,
			VK_OEM_102 = 0xE2,
			VK_ICO_HELP = 0xE3,
			VK_ICO_00 = 0xE4,
			VK_PROCESSKEY = 0xE5,
			VK_ICO_CLEAR = 0xE6,
			VK_PACKET = 0xE7,
			VK_OEM_RESET = 0xE9,
			VK_OEM_JUMP = 0xEA,
			VK_OEM_PA1 = 0xEB,
			VK_OEM_PA2 = 0xEC,
			VK_OEM_PA3 = 0xED,
			VK_OEM_WSCTRL = 0xEE,
			VK_OEM_CUSEL = 0xEF,
			VK_OEM_ATTN = 0xF0,
			VK_OEM_FINISH = 0xF1,
			VK_OEM_COPY = 0xF2,
			VK_OEM_AUTO = 0xF3,
			VK_OEM_ENLW = 0xF4,
			VK_OEM_BACKTAB = 0xF5,
			VK_ATTN = 0xF6,
			VK_CRSEL = 0xF7,
			VK_EXSEL = 0xF8,
			VK_EREOF = 0xF9,
			VK_PLAY = 0xFA,
			VK_ZOOM = 0xFB,
			VK_NONAME = 0xFC,
			VK_PA1 = 0xFD,
			VK_OEM_CLEAR = 0xFE
		}
		
		internal enum WindowMessage
		{
			WM_INPUT = 0x00FF,
			WM_SYSKEYDOWN = 0x0104,
			WM_SYSKEYUP = 0x0105,
			WM_CREATE = 0x0001,
			WM_KEYFIRST = 0x0100,
			WM_KEYDOWN = 0x0100,
			WM_KEYUP = 0x0101,
			WM_KEYLAST = 0x0109
		}

		internal enum GetRawInputDataCommand
		{
			RID_INPUT = 0x10000003,
			RID_HEADER = 0x10000005
		}
		
		internal enum RawInputDeviceInfoCommand
		{
			RIDI_PREPARSEDDATA = 0x20000005,
			RIDI_DEVICENAME = 0x20000007,
			RIDI_DEVICEINFO = 0x2000000b
		}
		
		internal struct RAWINPUTHEADER
		{
			public int dwType;
			public int dwSize;
			public int hDevice;
			public IntPtr wParam;
		}

		[StructLayout(LayoutKind.Explicit, Size=24)]	
			internal struct RAWMOUSE
		{
			[FieldOffset(0)] public short usFlags;
			[FieldOffset(2)] public RAWMOUSEUNION mouseUnion;
			//[FieldOffset(2)] public RAWMOUSEUNIONBUTTONS mouseUnionButtons;
			[FieldOffset(6)] public int ulRawButtons;
			[FieldOffset(12)] public int lLastX;
			[FieldOffset(16)] public int lLastY;
			[FieldOffset(20)] public int ulExtraInformation;
//			[FieldOffset(0)] public short usFlags;
//			[FieldOffset(2)] public int ulButtons;
//			[FieldOffset(2)] public short usButtonFlags;
//			[FieldOffset(4)] public short usButtonData;
//			[FieldOffset(8)] public int ulRawButtons;
//			[FieldOffset(12)] public int lLastX;
//			[FieldOffset(16)] public int lLastY;
//			[FieldOffset(20)] public int ulExtraInformation;
		}

		[StructLayout(LayoutKind.Explicit, Size=4)]
			internal struct RAWMOUSEUNION
		{
			[FieldOffset(0)] public short usButtonFlags;
			[FieldOffset(2)] public short usButtonData;
		}
		
		[StructLayout(LayoutKind.Explicit, Size=4)]
			internal struct RAWMOUSEUNIONBUTTONS
		{
			[FieldOffset(0)] public int ulButtons;
		}
		internal struct RAWKEYBOARD
		{
			public short MakeCode;
			public short Flags;
			public short Reserved;
			public short VKey;
			public int Message;
			public int ExtraInformation;
		}

		[StructLayout(LayoutKind.Explicit, Size=40)]	
			internal struct RAWINPUT
		{
			[FieldOffset(0)] public RAWINPUTHEADER header;
			[FieldOffset(16)] public RAWMOUSE mouse;
			[FieldOffset(16)] public RAWKEYBOARD keyboard;
		}
		
		[DllImport("user32.dll", SetLastError=true)]
		[return: MarshalAs(UnmanagedType.Bool)]
		internal static extern bool GetIconInfo(IntPtr hIcon, [In,Out] ref ICONINFO piconinfo);

		[DllImport("user32.dll", SetLastError=true)]
		[return: MarshalAs(UnmanagedType.Bool)]
		internal static extern bool DrawIconEx(IntPtr hdc, int xLeft, int yTop, IntPtr hIcon, int cxWidth, int cyWidth, int istepIfAniCur, IntPtr hbrFlickerFreeDraw, [MarshalAs(UnmanagedType.I4)] DrawIconFlag diFlags);
		
		[DllImport("user32.dll", SetLastError=true)]
		internal static extern int GetRawInputDeviceList(IntPtr intptrZero, [In,Out] ref int puiNumDevices, int cbSize);
		
		[DllImport("user32.dll", SetLastError=true)]
		internal static extern int GetRawInputDeviceList([Out] RAWINPUTDEVICELIST[] pRawInputDeviceList, [In,Out] ref int puiNumDevices, int cbSize);

		[DllImport("user32.dll", SetLastError=true)]
		internal static extern int GetRawInputDeviceInfo(IntPtr hDevice, int uiCommand, [In, Out] ref RID_DEVICE_INFO pData, [In,Out] ref int pcbSize);

		[DllImport("user32.dll", SetLastError=true)]
		internal static extern bool RegisterRawInputDevices(RAWINPUTDEVICE[] pRawInputDevices, int uiNumDevices, int cbSize);

		[DllImport("user32.dll", SetLastError=true)]
		internal static extern int ShowCursor(bool bShow);

		[DllImport("user32.dll", SetLastError=true)]
		internal unsafe static extern int GetRawInputData(RAWINPUT* hRawInput, int uiCommand, [In, Out] ref RAWINPUT pData, [In, Out] ref int pcbSize, int cbSizeHeader);

		[DllImport("user32.dll", SetLastError=true)]
		internal static extern bool SetCursorPos(int X, int Y);

		[DllImport("user32.dll", SetLastError=true)]
		internal static extern short GetKeyState(int nVirtKey);
		

	}
	
}
